@@ -1,10 +1,9 @@
-#!/usr/local/bin/perl
 #
 #	XS.pm : perl function definition for Term::ReadLine::Gnu
 #
-#	$Id: XS.pm,v 1.20 2002-07-27 22:39:49-05 hiroo Exp $
+#	$Id: XS.pm 556 2016-11-03 14:24:45Z hayashi $
 #
-#	Copyright (c) 2002 Hiroo Hayashi.  All rights reserved.
+#	Copyright (c) 1999-2016 Hiroo Hayashi.  All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or
 #	modify it under the same terms as Perl itself.
@@ -13,18 +12,22 @@ package Term::ReadLine::Gnu::XS;
 
 use Carp;
 use strict;
+use warnings;
 use AutoLoader 'AUTOLOAD';
 
+our $VERSION;
+$VERSION='1.35';	# added for CPAN
+
 # make aliases
-use vars qw(%Attribs);
+our %Attribs;
 *Attribs = \%Term::ReadLine::Gnu::Attribs;
 
-use vars qw(*read_history);
+#use vars qw(*read_history);
 *read_history = \&read_history_range;
 
 # alias for 8 characters limitation imposed by AutoSplit
-use vars qw(*rl_unbind_key *rl_unbind_function *rl_unbind_command
-	    *history_list *history_arg_extract);
+#use vars qw(*rl_unbind_key *rl_unbind_function *rl_unbind_command
+#	    *history_list *history_arg_extract);
 *rl_unbind_key = \&unbind_key;
 *rl_unbind_function = \&unbind_function;
 *rl_unbind_command = \&unbind_command;
@@ -32,7 +35,7 @@ use vars qw(*rl_unbind_key *rl_unbind_function *rl_unbind_command
 *history_arg_extract = \&hist_arg_extract;
 
 # For backward compatibility.  Using these name (*_in_map) is deprecated.
-use vars qw(*rl_unbind_function_in_map *rl_unbind_command_in_map);
+#use vars qw(*rl_unbind_function_in_map *rl_unbind_command_in_map);
 *rl_unbind_function_in_map = \&unbind_function;
 *rl_unbind_command_in_map  = \&unbind_command;
 
@@ -62,8 +65,10 @@ my $_tstrs_init = 0;
 # Uncomment the following line to enable AutoSplit.  If you are using
 # AutoLoader.pm distributed with Perl 5.004 or earlier, you must
 # update AutoLoader.pm due to its bug.
+# 2016/06/07: now it is time to uncomment. Ignore the warning message,
+# "... some names are not unique ...", during make.
 
-#__END__
+__END__
 
 
 #
@@ -87,6 +92,7 @@ sub rl_copy_keymap ($)    { return _rl_copy_keymap(_str2map($_[0])); }
 sub rl_discard_keymap ($) { return _rl_discard_keymap(_str2map($_[0])); }
 sub rl_set_keymap ($)     { return _rl_set_keymap(_str2map($_[0])); }
 
+# rl_bind_key
 sub rl_bind_key ($$;$) {
     if (defined $_[2]) {
 	return _rl_bind_key($_[0], _str2fn($_[1]), _str2map($_[2]));
@@ -95,6 +101,21 @@ sub rl_bind_key ($$;$) {
     }
 }
 
+# rl_bind_key_if_unbound
+sub rl_bind_key_if_unbound ($$;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 5.0) {
+	carp "rl_bind_key_if_unbound() is not supported.  Ignored\n";
+	return;
+    }
+    if (defined $_[2]) {
+	return _rl_bind_key_if_unbound($_[0], _str2fn($_[1]), _str2map($_[2]));
+    } else {
+	return _rl_bind_key_if_unbound($_[0], _str2fn($_[1]));
+    }
+}
+
 # rl_unbind_key
 sub unbind_key ($;$) {
     if (defined $_[1]) {
@@ -135,6 +156,21 @@ sub unbind_command ($;$) {
     }
 }
 
+# rl_bind_keyseq
+sub rl_bind_keyseq ($$;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 5.0) {
+	carp "rl_bind_keyseq() is not supported.  Ignored\n";
+	return;
+    }
+    if (defined $_[2]) {
+	return _rl_bind_keyseq($_[0], _str2fn($_[1]), _str2map($_[2]));
+    } else {
+	return _rl_bind_keyseq($_[0], _str2fn($_[1]));
+    }
+}
+
 sub rl_set_key ($$;$) {
     my ($version) = $Attribs{library_version}
 	=~ /(\d+\.\d+)/;
@@ -149,6 +185,21 @@ sub rl_set_key ($$;$) {
     }
 }
 
+# rl_bind_keyseq_if_unbound
+sub rl_bind_keyseq_if_unbound ($$;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 5.0) {
+	carp "rl_bind_keyseq_if_unbound() is not supported.  Ignored\n";
+	return;
+    }
+    if (defined $_[2]) {
+	return _rl_bind_keyseq_if_unbound($_[0], _str2fn($_[1]), _str2map($_[2]));
+    } else {
+	return _rl_bind_keyseq_if_unbound($_[0], _str2fn($_[1]));
+    }
+}
+
 sub rl_macro_bind ($$;$) {
     my ($version) = $Attribs{library_version}
 	=~ /(\d+\.\d+)/;
@@ -212,6 +263,12 @@ sub rl_add_funmap_entry ($$) {
 }
 
 sub rl_tty_set_default_bindings (;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 4.2) {
+	carp "rl_tty_set_default_bindings() is not supported.  Ignored\n";
+	return;
+    }
     if (defined $_[0]) {
 	return _rl_tty_set_defaut_bindings(_str2map($_[1]));
     } else {
@@ -219,6 +276,20 @@ sub rl_tty_set_default_bindings (;$) {
     }
 }
 
+sub rl_tty_unset_default_bindings (;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 5.0) {
+	carp "rl_tty_unset_default_bindings() is not supported.  Ignored\n";
+	return;
+    }
+    if (defined $_[0]) {
+	return _rl_tty_unset_defaut_bindings(_str2map($_[1]));
+    } else {
+	return _rl_tty_unset_defaut_bindings();
+    }
+}
+
 sub rl_message {
     my $fmt = shift;
     my $line = sprintf($fmt, @_);
@@ -291,8 +362,7 @@ sub get_history_event ( $$;$ ) {
 # Readline Library already uses, instead of Term::Caps.pm.
 
 # Some terminals do not support 'ue' (underline end).
-use vars qw(%term_no_ue);
-%term_no_ue = ( kterm => 1 );
+our %term_no_ue = ( kterm => 1 );
 
 sub ornaments {
     return $rl_term_set unless @_;
@@ -495,7 +565,10 @@ sub _trp_completion_function ( $$ ) {
     my($text, $state) = @_;
 
     my $cf;
-    return undef unless defined ($cf = $Attribs{completion_function});
+    if (not defined ($cf = $Attribs{completion_function})) {
+	carp "_trp_comletion_fuction: internal error\n";
+	return undef;
+    }
 
     if ($state) {
 	$_i++;
@@ -511,7 +584,10 @@ sub _trp_completion_function ( $$ ) {
     }
 
     for (; $_i <= $#_matches; $_i++) {
-	return $_matches[$_i] if ($_matches[$_i] =~ /^\Q$text/);
+	# case insensitive match to be compatible with
+	# Term::ReadLine::Perl.
+	# https://rt.cpan.org/Ticket/Display.html?id=72378
+	return $_matches[$_i] if ($_matches[$_i] =~ /^\Q$text/i);
     }
     return undef;
 }
